//=======================================================================
//                        MicroBoard LX9 USBHOST
//
// File:
//   fm_8b/10b.v
//
// Abstract:
//   DVI 8b/10b encoding
//  Created:
//    11 March 2014
//======================================================================
//
// Copyright (c) 2014, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Revision History
// $Date: 2014-04-03 13:30:47 +0900 (Thu, 03 Apr 2014) $
// $Rev: 50 $

module fm_8b10b (
  clk_pix,
  rst_x,
  i_d,
  i_c0,
  i_c1,
  i_de,
  o_q
);

////////////////////////////
// I/O definition
////////////////////////////
input         clk_pix;
input         rst_x;
input  [7:0]  i_d;
input         i_c0;
input         i_c1;
input         i_de;
output [9:0]  o_q;
/////////////////////////
//  Register definition
/////////////////////////
// input register
reg  [7:0]  r_d;
reg         r_c0;
reg         r_c1;
reg         r_de;

reg [8:0] w_q_m;
reg [9:0] w_q_out;
reg [9:0] r_q_out;
reg [7:0] w_cnt;
reg [7:0] r_cnt;
/////////////////////////
//  wire definition
/////////////////////////
wire   w_condition0;
wire   w_condition1;
wire   w_condition2;
// /////////////////////////
//  assign statement
/////////////////////////
// (N1{D}>4) OR (N1{D} == 4 AND D[0] == 0)
assign w_condition0 = (f_n1(r_d) > 'd4) |
                      ((f_n1(r_d) == 'd4)&(~r_d[0]));

// (Cnt(t-1)==0) OR (N1{q_m[0:7]}==N0[q_m[0:7]])
assign w_condition1 = (r_cnt == 8'h00) |
                      (f_n1(w_q_m[7:0]) == f_n0(w_q_m[7:0]));

// (Cnt(t-1)>0) AND (N1{q_m[0:7]}>N0[q_m[0:7]]) OR
// (Cnt(t-1)<0) AND (N0{q_m[0:7]}>N1[q_m[0:7]])
assign w_condition2 = ((~r_cnt[7]) &
                      (f_n1(w_q_m[7:0]) > f_n0(w_q_m[7:0]))) |
                      ((r_cnt[7]) &
                      (f_n0(w_q_m[7:0]) > f_n1(w_q_m[7:0])));

assign o_q = r_q_out;
////////////////////////
// always 
///////////////////////

  always @(posedge clk_pix or negedge rst_x) begin
    if (~rst_x) begin
      r_d <= 8'h00;
      r_c0 <= 1'b0;
      r_c1 <= 1'b0;
      r_de <= 1'b0;
    end else begin
      r_d <= i_d;
      r_c0 <= i_c0;
      r_c1 <= i_c1;
      r_de <= i_de;
    end
  end

  // data stream disparity
  always @(posedge clk_pix or negedge rst_x) begin
    if (~rst_x) begin
      r_cnt <= 'b11111111; // negative
    end else begin
      r_cnt <= w_cnt;
    end
  end

  always @(posedge clk_pix) begin
    r_q_out <= w_q_out;
  end

  // 8b/10b encoding
  always @* begin
    if (w_condition0) begin
      w_q_m[0] = r_d[0];
      w_q_m[1] = w_q_m[0] ~^ r_d[1];
      w_q_m[2] = w_q_m[1] ~^ r_d[2];
      w_q_m[3] = w_q_m[2] ~^ r_d[3];
      w_q_m[4] = w_q_m[3] ~^ r_d[4];
      w_q_m[5] = w_q_m[4] ~^ r_d[5];
      w_q_m[6] = w_q_m[5] ~^ r_d[6];
      w_q_m[7] = w_q_m[6] ~^ r_d[7];
      w_q_m[8] = 1'b0;
    end else begin
      w_q_m[0] = r_d[0];
      w_q_m[1] = w_q_m[0] ^ r_d[1];
      w_q_m[2] = w_q_m[1] ^ r_d[2];
      w_q_m[3] = w_q_m[2] ^ r_d[3];
      w_q_m[4] = w_q_m[3] ^ r_d[4];
      w_q_m[5] = w_q_m[4] ^ r_d[5];
      w_q_m[6] = w_q_m[5] ^ r_d[6];
      w_q_m[7] = w_q_m[6] ^ r_d[7];
      w_q_m[8] = 1'b1;
    end
    if (~r_de) begin
      w_cnt = 8'h00;
      case ({r_c1,r_c0})
        2'b00: w_q_out[9:0] = 10'b1101010100;
        2'b01: w_q_out[9:0] = 10'b0010101011;
        2'b10: w_q_out[9:0] = 10'b0101010100;
        2'b11: w_q_out[9:0] = 10'b1010101011;
      endcase
    end else begin
      if (w_condition1) begin
        w_q_out[9] = ~w_q_m[8];
        w_q_out[8] =  w_q_m[8];
        w_q_out[7:0] =  (w_q_m[8]) ? w_q_m[7:0] : ~w_q_m[7:0];
        if (w_q_m[8]) begin
          w_cnt = r_cnt + (f_n1(w_q_m[7:0])-f_n0(w_q_m[7:0]));
        end else begin
          w_cnt = r_cnt + (f_n0(w_q_m[7:0])-f_n1(w_q_m[7:0]));
        end
      end else begin
        if (w_condition2) begin
          w_q_out[9] = 1'b1;
          w_q_out[8] =  w_q_m[8];
          w_q_out[7:0] = ~w_q_m[7:0];
          w_cnt = r_cnt + {6'b0,w_q_m[8],1'b0} + (f_n0(w_q_m[7:0])-f_n1(w_q_m[7:0]));
        end else begin
          w_q_out[9] = 1'b0;
          w_q_out[8] =  w_q_m[8];
          w_q_out[7:0] = w_q_m[7:0];
          w_cnt = r_cnt - {6'b0,~w_q_m[8],1'b0} + (f_n1(w_q_m[7:0])-f_n0(w_q_m[7:0]));
        end
      end
    end
  end


////////////////////////
// function
///////////////////////
function [3:0] f_n1;
  input [7:0] di;
  reg [3:0] n;
  integer i;
  begin
    n = 4'd0;
    for (i=0;i<8;i=i+1) begin
      if (di[i]) n = n + 1'b1;
    end
    f_n1 = n;
  end
endfunction

function [3:0] f_n0;
  input [7:0] di;
  reg [3:0] n;
  integer i;
  begin
    n = 4'd0;
    for (i=0;i<8;i=i+1) begin
      if (~di[i]) n = n + 1'b1;
    end
    f_n0 = n;
  end
endfunction

endmodule


