//=======================================================================
//                        MicroBoard LX9 USBHOST
//
// File:
//   fm_dvi.v
//
// Abstract:
//   DVI encoder
//  Created:
//    11 March 2014
//======================================================================
//
// Copyright (c) 2014, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Revision History
// $Date: 2014-04-03 13:30:47 +0900 (Thu, 03 Apr 2014) $
// $Rev: 50 $

module fm_dvi (
  clk_pix,
  clk_pix_x5,
  rst_x,
  i_cr,
  i_cg,
  i_cb,
  i_vs,
  i_hs,
  i_de,
  o_d_p,
  o_d_n,
  o_c_p,
  o_c_n
);

////////////////////////////
// I/O definition
////////////////////////////
input         clk_pix;
input         clk_pix_x5;
input         rst_x;
input  [7:0]  i_cr;
input  [7:0]  i_cg;
input  [7:0]  i_cb;
input         i_vs;
input         i_hs;
input         i_de;
output [2:0]  o_d_p;
output [2:0]  o_d_n;
output        o_c_p;
output        o_c_n;

/////////////////////////
//  wire
/////////////////////////
wire [9:0] w_q_r;
wire [9:0] w_q_g;
wire [9:0] w_q_b;

wire w_s_r_l;
wire w_s_r_h;
wire w_s_g_l;
wire w_s_g_h;
wire w_s_b_l;
wire w_s_b_h;
wire w_s_c_l;
wire w_s_c_h;
   
/////////////////////////
//  module instance
/////////////////////////

fm_8b10b u_8b10b_b (
  .clk_pix(clk_pix),
  .rst_x(rst_x),
  .i_d(i_cb),
  .i_c0(i_hs),
  .i_c1(i_vs),
  .i_de(i_de),
  .o_q(w_q_b)
);

fm_8b10b u_8b10b_g (
  .clk_pix(clk_pix),
  .rst_x(rst_x),
  .i_d(i_cg),
  .i_c0(1'b0),
  .i_c1(1'b0),
  .i_de(i_de),
  .o_q(w_q_g)
);

fm_8b10b u_8b10b_r (
  .clk_pix(clk_pix),
  .rst_x(rst_x),
  .i_d(i_cr),
  .i_c0(1'b0),
  .i_c1(1'b0),
  .i_de(i_de),
  .o_q(w_q_r)
);

// CH0: Blue 
fm_serialize u_serialize_ch0 (
  .clk_pix(clk_pix),
  .clk_pix_x5(clk_pix_x5),
  .rst_x(rst_x),
  .i_d(w_q_b),
  .o_s_l(w_s_b_l),
  .o_s_h(w_s_b_h)
);

// CH1: Green
fm_serialize u_serialize_ch1 (
  .clk_pix(clk_pix),
  .clk_pix_x5(clk_pix_x5),
  .rst_x(rst_x),
  .i_d(w_q_g),
  .o_s_l(w_s_g_l),
  .o_s_h(w_s_g_h)
);

// CH2: Red
fm_serialize u_serialize_ch2 (
  .clk_pix(clk_pix),
  .clk_pix_x5(clk_pix_x5),
  .rst_x(rst_x),
  .i_d(w_q_r),
  .o_s_l(w_s_r_l),
  .o_s_h(w_s_r_h)
);

// CH3: Clock
fm_serialize u_serialize_ch3 (
  .clk_pix(clk_pix),
  .clk_pix_x5(clk_pix_x5),
  .rst_x(rst_x),
  .i_d(10'b0000011111),
  .o_s_l(w_s_c_l),
  .o_s_h(w_s_c_h)
);

// I/O Buffer
fm_ddio u_ddio_ch0 (
  .clk(clk_pix_x5),
  .rst_x(rst_x),
  .i_data_h(w_s_b_h),
  .i_data_l(w_s_b_l),
  .o_data_p(o_d_p[0]),
  .o_data_n(o_d_n[0])
);

fm_ddio u_ddio_ch1 (
  .clk(clk_pix_x5),
  .rst_x(rst_x),
  .i_data_h(w_s_g_h),
  .i_data_l(w_s_g_l),
  .o_data_p(o_d_p[1]),
  .o_data_n(o_d_n[1])
);

fm_ddio u_ddio_ch2 (
  .clk(clk_pix_x5),
  .rst_x(rst_x),
  .i_data_h(w_s_r_h),
  .i_data_l(w_s_r_l),
  .o_data_p(o_d_p[2]),
  .o_data_n(o_d_n[2])
);

fm_ddio u_ddio_ch3 (
  .clk(clk_pix_x5),
  .rst_x(rst_x),
  .i_data_h(w_s_c_h),
  .i_data_l(w_s_c_l),
  .o_data_p(o_c_p),
  .o_data_n(o_c_n)
);

endmodule


