//=======================================================================
//                        MicroBoard LX9 USBHOST
//
// File:
//    usbhost_top.v
//
// Abstract:
//    OpenCores usb host implementation top module
//
// Created:
//    29 June 2013
//
//======================================================================
//
// Copyright (c) 2013, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//  Revision History
// $Date: 2014-04-03 13:36:47 +0900 (Thu, 03 Apr 2014) $
// $Rev: 51 $

module usbhost_top
  (
  input  CLK_100MHZ,
  input  CLK_66MHZ,
  input  CLK_40MHZ,
  input  USER_RESET,
  output [3:0] LED,
  input UART_Rx,
  output UART_Tx,
  inout USB_Dp,
  inout USB_Dn,
  // LPDDR
  inout  [51:0] mcb3_dram_dq,
  output [12:0] mcb3_dram_a,
  output [1:0] mcb3_dram_ba,
  output mcb3_dram_cke,
  output mcb3_dram_ras_n,
  output mcb3_dram_cas_n,
  output mcb3_dram_we_n,
  output mcb3_dram_dm,
  inout  mcb3_dram_udqs,
  inout  mcb3_rzq,
  output mcb3_dram_udm,
  inout  mcb3_dram_dqs,
  output mcb3_dram_ck,
  output mcb3_dram_ck_n,
  // DVI
  output FPGA_PMOD2_P1, // out   [MBEC] scl
  inout  FPGA_PMOD2_P2,	// inout [MBEC] sda
  input	 FPGA_PMOD2_P3,	// in    [MBEC] hpd
  inout	 FPGA_PMOD2_P4,	// inout [MBEC] lvds/#tmds
  output FPGA_PMOD2_P7, // out   [MBEC] tx0 SSTL3_II
  output FPGA_PMOD2_P8, // out   [MBEC] tx1 SSTL3_II
  output FPGA_PMOD2_P9, // out   [MBEC] tx2 SSTL3_II
  output FPGA_PMOD2_P10 // out   [MBEC] txc SSTL3_II
  );

  //******************************************************************//
  // Create clock.                                                    //
  //******************************************************************//

  wire  clk_core;
  wire  clk48;
  wire  clk25;
  wire  clk125;  // 25 x 5
  wire  clk_enable;
  wire  pll_locked;
  wire  rst_x;
  wire  rst;
// microblaze_mcs
  wire w_IO_Read_Strobe;
  wire w_IO_Addr_Strobe;
  wire w_IO_Ready;
  wire w_IO_Write_Strobe;
  wire [31:0] w_IO_Address;
  wire [3:0]  w_IO_Byte_Enable;
  wire [31:0] w_IO_Write_Data;
  wire [31:0] w_IO_Read_Data;

  wire w_c3_calib_done;

// bridge
  wire        w_req;
  wire        w_wr;
  wire [31:0] w_adrs;
  wire        w_ack;
  wire [3:0]  w_be;
  wire [31:0] w_wd;
  wire        w_rstr;
  wire [31:0] w_rd;
// system register
  wire w_sys_strobe;
  wire w_sys_wr;
  wire [21:0] w_sys_adrs;
  wire [3:0] w_sys_be;
  wire [31:0] w_sys_wd;
  wire [31:0] w_sys_rd;
  wire w_sys_ack;

// ddr access from cpu
  wire w_ddr_strobe;
  wire w_ddr_wr;
  wire [21:0] w_ddr_adrs;
  wire [3:0] w_ddr_be;
  wire [31:0] w_ddr_wd;
  wire [31:0] w_ddr_rd;
  wire w_ddr_ack;
// display controller
  wire w_vint_x;
  wire w_vint_edge;
  wire [7:0] w_cr;
  wire [7:0] w_cg;
  wire [7:0] w_cb;
  wire w_vsync_x;
  wire w_hsync_x;
  wire w_blank_x;
  wire w_de;
  wire w_req_dc;
  wire [21:0] w_adrs_dc;
  wire [5:0] w_len_dc;
  wire w_ack_dc;
  wire w_rstr_dc;
  wire [31:0] w_rd_dc;
  // USB host
  wire           w_usb_strobe;
  wire           w_usb_we;
  wire  [7:0]    w_usb_adrs;
  wire  [7:0]    w_usb_wd;
  wire  [7:0]    w_usb_rd;
  wire           w_usb_ack;
  wire           w_host_sof_int;
  wire           w_host_connect_int;
  wire           w_host_resume_int;
  wire           w_host_trans_int;
  wire  [1:0]    w_host_usb_wire_data_in;
  wire  [1:0]    w_host_usb_wire_data_out;
  wire           w_host_usb_wire_ctrl;
  wire           w_usb_fullspeed;
  assign USB_Dp = (w_host_usb_wire_ctrl) ? w_host_usb_wire_data_out[1] : 1'hz;
  assign USB_Dn = (w_host_usb_wire_ctrl) ? w_host_usb_wire_data_out[0] : 1'hz;
  assign w_host_usb_wire_data_in[0] = USB_Dn;
  assign w_host_usb_wire_data_in[1] = USB_Dp;

  assign rst_x = ~rst;
  assign FPGA_PMOD2_P1 = 1'bz;
  assign FPGA_PMOD2_P2 = 1'bz;
  assign FPGA_PMOD2_P4 = 1'h0;

  wire w_video_start;
  wire [3:0] w_fb0_offset;
  wire [3:0] w_fb0_ms_offset;
  wire [3:0] w_fb1_offset;
  wire [3:0] w_fb1_ms_offset;
  wire [1:0] w_color_mode;
  wire w_front_buffer;
   
  wire w_c3_p0_rd_empty;
  wire w_c3_p0_cmd_full;
  wire w_c3_p0_wr_full;
  wire w_int;
  wire w_FIT1_Toggle;
clk_wiz_v3_6 clk_wiz (
  .CLK_IN1(CLK_100MHZ),
  // Clock out ports
  .CLK_OUT1(clk_core),
  .CLK_OUT2(clk25),
  .CLK_OUT3(clk125),
  // Status and control signals
  .RESET(USER_RESET),
  .LOCKED(pll_locked)
);
   
clk_wiz_v3_6_usb clk_wiz_usbp (
  .CLK_IN1(CLK_40MHZ),
  // Clock out ports
  .CLK_OUT1(clk48),
  // Status and control signals
  .RESET(USER_RESET),
  .LOCKED()
);

microblaze_mcs_v1_4 mcs_0 (
  .Clk(clk_core),
  .Reset(~rst_x),
  .UART_Tx(UART_Tx),
  .UART_Rx(UART_Rx),
  .GPI1_Interrupt(),
  .GPI1({4'ha,
         w_c3_p0_rd_empty,
         w_c3_p0_cmd_full,
         /*w_c3_p0_wr_full,
           w_c3_calib_done*/
			  w_vint_x,
			  w_int
			  }),
  .FIT1_Interrupt(),
  .FIT1_Toggle(w_FIT1_Toggle),
  .INTC_Interrupt(w_int),
  .FIT2_Interrupt(),
  .FIT2_Toggle(w_FIT1_Toggle),
  .IO_Addr_Strobe(w_IO_Addr_Strobe),
  .IO_Read_Strobe(w_IO_Read_Strobe),
  .IO_Write_Strobe(w_IO_Write_Strobe),
  .IO_Ready(w_IO_Ready),
  .IO_Read_Data(w_IO_Read_Data),
  .IO_Address(w_IO_Address),
  .IO_Byte_Enable(w_IO_Byte_Enable),
  .IO_Write_Data(w_IO_Write_Data)
);
  
fm_mbbridge u_mbbridge (
  .clk_core(clk_core),
  .rst_x(rst_x),
  // microblaze side
  .IO_Addr_Strobe(w_IO_Addr_Strobe),
  .IO_Read_Strobe(w_IO_Read_Strobe),
  .IO_Write_Strobe(w_IO_Write_Strobe),
  .IO_Ready(w_IO_Ready),
  .IO_Read_Data(w_IO_Read_Data),
  .IO_Address(w_IO_Address),
  .IO_Byte_Enable(w_IO_Byte_Enable),
  .IO_Write_Data(w_IO_Write_Data),
  // internal side
  .o_req(w_req),
  .o_wr(w_wr),
  .o_adrs(w_adrs),
  .i_ack(w_ack),
  .o_be(w_be),
  .o_wd(w_wd),
  .i_rstr(w_rstr),
  .i_rd(w_rd)
);

fm_dispatch_io u_dispatch_io (
  .clk_core(clk_core),
  .rst_x(rst_x),
  //local interface
  .i_req(w_req),
  .i_wr(w_wr),
  .i_adrs(w_adrs),
  .o_ack(w_ack),
  .i_be(w_be),
  .i_wd(w_wd),
  .o_rstr(w_rstr),
  .o_rd(w_rd),
  // usb wishbone
  .o_usb_strobe(w_usb_strobe),
  .o_usb_we(w_usb_we),
  .o_usb_adrs(w_usb_adrs),
  .o_usb_wd(w_usb_wd),
  .i_usb_rd(w_usb_rd),
  .i_usb_ack(w_usb_ack),
  // system register
  .o_sys_strobe(w_sys_strobe),
  .o_sys_wr(w_sys_wr),
  .o_sys_adrs(w_sys_adrs),
  .o_sys_be(w_sys_be),
  .o_sys_wd(w_sys_wd),
  .i_sys_rstr(w_sys_rstr),
  .i_sys_rd(w_sys_rd),
  .i_sys_ack(w_sys_ack),
  // lpddr	
  .o_ddr_strobe(w_ddr_strobe),
  .o_ddr_wr(w_ddr_wr),
  .o_ddr_adrs(w_ddr_adrs),
  .o_ddr_be(w_ddr_be),
  .o_ddr_wd(w_ddr_wd),
  .i_ddr_rstr(w_ddr_rstr),
  .i_ddr_rd(w_ddr_rd),
  .i_ddr_ack(w_ddr_ack)
);

// System controller
fm_sys u_sys (
  .clk_core(clk_core),
  .rst_x(rst_x),
  // internal interface
  .i_req(w_sys_strobe),
  .i_wr(w_sys_wr),
  .i_adrs(w_sys_adrs),
  .o_ack(w_sys_ack),
  .i_be(w_sys_be),
  .i_wd(w_sys_wd),
  .o_rstr(w_sys_rstr),
  .o_rd(w_sys_rd),
  // configuration output
  //   Video controller
  .o_video_start(w_video_start),
  .o_aa_en(),
  .o_fb0_offset(w_fb0_offset),
  .o_fb0_ms_offset(w_fb0_ms_offset),
  .o_fb1_offset(w_fb1_offset),
  .o_fb1_ms_offset(w_fb1_ms_offset),
  .o_color_mode(w_color_mode),
  .o_front_buffer(w_front_buffer),
  .o_fb_blend_en(),
  // vint
  .i_vint_x(w_vint_x),
  .i_vint_edge(w_vint_edge),
  // vertex dma int
  .i_vtx_int(1'b0),
  // int out
  .o_int(w_int)
);

// Video controller
fm_hvc u_hvc (
  .clk_core(clk_core),
  .clk_vi(clk25),
  .rst_x(rst_x),
  // configuration registers
  .i_video_start(w_video_start),
  .i_fb0_offset(w_fb0_offset),
  .i_fb0_ms_offset(w_fb0_ms_offset),
  .i_fb1_offset(w_fb1_offset),
  .i_fb1_ms_offset(w_fb1_ms_offset),
  .i_color_mode(w_color_mode),
  .i_front_buffer(w_front_buffer),
  .i_aa_en('d0),
  .i_fb_blend_en(1'b0),
  // status out
  .o_vint_x(w_vint_x),
  .o_vint_edge(w_vint_edge),
  // dram if
  .o_req(w_req_dc),
  .o_adrs(w_adrs_dc),
  .o_len(w_len_dc),
  .i_ack(w_ack_dc),
  .i_rstr(w_rstr_dc),
  .i_rd(w_rd_dc),
  // video out
  .clk_vo(),
  .o_r(w_cr),
  .o_g(w_cg),
  .o_b(w_cb),
  .o_vsync_x(w_vsync_x),
  .o_hsync_x(w_hsync_x),
  .o_blank_x(w_blank_x),
  .o_de(w_de)
);

usbHost u_usbHost (
  .clk_i(clk_core),
  .rst_i(~rst_x),
  .address_i(w_usb_adrs),
  .data_i(w_usb_wd),
  .data_o(w_usb_rd),
  .we_i(w_usb_we),
  .strobe_i(w_usb_strobe),
  .ack_o(w_usb_ack),
  .usbClk(clk48),
  .hostSOFSentIntOut(w_host_sof_int),
  .hostConnEventIntOut(w_host_connect_int),
  .hostResumeIntOut(w_host_resume_int),
  .hostTransDoneIntOut(w_host_trans_int),
  .USBWireDataIn(w_host_usb_wire_data_in),
  .USBWireDataInTick(),
  .USBWireDataOut(w_host_usb_wire_data_out),
  .USBWireDataOutTick(),
  .USBWireCtrlOut(w_host_usb_wire_ctrl),
  .USBFullSpeed(w_usb_fullspeed)
);

 mig_wrapper u_mig_wrapper (
  // LPDDR
  .mcb3_dram_dq(mcb3_dram_dq),
  .mcb3_dram_a(mcb3_dram_a),
  .mcb3_dram_ba(mcb3_dram_ba),
  .mcb3_dram_cke(mcb3_dram_cke),
  .mcb3_dram_ras_n(mcb3_dram_ras_n),
  .mcb3_dram_cas_n(mcb3_dram_cas_n),
  .mcb3_dram_we_n(mcb3_dram_we_n),
  .mcb3_dram_dm(mcb3_dram_dm),
  .mcb3_dram_udqs(mcb3_dram_udqs),
  .mcb3_rzq(mcb3_rzq),
  .mcb3_dram_udm(mcb3_dram_udm),
  .c3_sys_clk(CLK_66MHZ),
  .c3_sys_rst_i(~pll_locked),
  .c3_calib_done(w_c3_calib_done),
  .c3_clk0(),
  .c3_rst0(rst),
  .mcb3_dram_dqs(mcb3_dram_dqs),
  .mcb3_dram_ck(mcb3_dram_ck),
  .mcb3_dram_ck_n(mcb3_dram_ck_n),
  // cpu access
  .clk_core(clk_core),
  .rst_x(rst_x),
  // cpu access
  .i_req_cpu(w_ddr_strobe),
  .i_wr_cpu(w_ddr_wr),
  .i_adrs_cpu(w_ddr_adrs),
  .o_ack_cpu(w_ddr_ack),
  .i_be_cpu(w_ddr_be),
  .i_wd_cpu(w_ddr_wd),
  .o_rstr_cpu(w_ddr_rstr),
  .o_rd_cpu(w_ddr_rd),
  // dc access
  .i_req_dc(w_req_dc),
  .i_adrs_dc(w_adrs_dc),
  .i_len_dc(w_len_dc),
  .o_ack_dc(w_ack_dc),
  .o_rstr_dc(w_rstr_dc),
  .o_rd_dc(w_rd_dc),
  // debug
  .c3_p0_rd_empty(w_c3_p0_rd_empty),
  .c3_p0_cmd_full(w_c3_p0_cmd_full),
  .c3_p0_wr_full(w_c3_p0_wr_full)
			    
);

fm_dvi u_dvi (
  .clk_pix(clk25),
  .clk_pix_x5(clk125),
  .rst_x(rst_x),
  .i_cr(w_cr),
  .i_cg(w_cg),
  .i_cb(w_cb),
  .i_vs(w_vsync_x),
  .i_hs(w_hsync_x),
  .i_de(w_de),
  .o_d_p({FPGA_PMOD2_P7,
          FPGA_PMOD2_P8,
          FPGA_PMOD2_P9}),
  .o_d_n(),
  .o_c_p(FPGA_PMOD2_P10),
  .o_c_n()
);

  //******************************************************************//
  //                                                                  //
  //******************************************************************//

  reg  [26:0] led_count;

  always @(posedge clk_core) 
  	led_count <= led_count + 1;
  
  //assign LED[3:0] = led_count[26:23];	// connects led outputs to counter value
  assign LED[3:2] = led_count[24:23];	// connects led outputs to counter value
  assign LED[0] = w_c3_calib_done;
  assign LED[1] = pll_locked;
endmodule
