/*
 * USB Host example
 *   HID: mouse, keyboard, gamepad
 *   MSC Bulk-only transfer with Petit FatFs
 */

#include <stdio.h>
#include <string.h>
#include "platform.h"
#include "usbh_hw_reg.h"
#include "usbh_env.h"
#include "pff.h"
#include "video.h"

st_hid_gamepad gamepad_data;
un_hid_mouse mouse_data;
un_hid_keyboard keyboard_data;
#define keyboard_buffer (*(volatile char *)(KEY_BUFFER)

volatile int fit_status;
volatile int video_status;
volatile int fb_front;
#define IRQ_STATUS (*(volatile unsigned int *)(0x80000030))
#define IRQ_ENABLE (*(volatile unsigned int *)(0x80000038))
#define IRQ_ACK (*(volatile unsigned int *)(0x8000003c))

void fit_func() {
    int status;
	microblaze_disable_interrupts();
	status = IRQ_STATUS;
	if (status & 0x80) {
	  fit_status = 1;// fit
	  IRQ_ACK = 0x00000080;
	}
	if (status & 0x10000) {
	  video_status = 1;// video
	  if (VIDEO_STATUS) VIDEO_STATUS = 0;
	  while (VIDEO_STATUS &1);
	  IRQ_ACK = 0x00010000;
	}
	microblaze_enable_interrupts();
}
// keyboard call back
void on_keyboard(int key, int modifier) {
  int len;
  volatile char *kb;
  kb = (volatile char *)(KEY_BUFFER);
  len = strlen(kb);
  xil_printf("len= %d\n",len);
  if (key == 0x7f) {
    if (len >0) kb[len-1] = 0;
  } else {
	 if (len <12) {
     kb[len] = key;
	 kb[len+1] = 0;
	 }
  }
  xil_printf("key changed %c %x\r\n",key,modifier);
  keyboard_data.BYTE.modifier = modifier;
  keyboard_data.BYTE.keycode[0] = key;
}
// gamepad callback
void on_gamepad(int buttons, int x, int y) {
  static int initial_flag = 1;
  static int pb = 0, px = 0, py = 0;
  if (!initial_flag) {
    if ((pb != buttons)||(px != x)||(py!=y))
      xil_printf("gamepad changed %x %x %x\r\n",buttons,x,y);
  }

  initial_flag =0;
  pb = buttons; px = x; py = y;
  gamepad_data.BYTE.x = x;
  gamepad_data.BYTE.y = y;
  gamepad_data.BYTE.b0.BYTE =buttons;
}
// mouse callback
void on_mouse(int buttons, int x, int y, int z) {
  mouse_data.BYTE.x = x;
  mouse_data.BYTE.y = y;
  mouse_data.BYTE.b = buttons;
  xil_printf("mouse changed %x %x %x %x\r\n",buttons,x,y,z);
}

FATFS fatfs;// File system object
int main()
{
    volatile char *kb;
    int i;
    init_platform();

    microblaze_register_handler((XInterruptHandler)fit_func,(void*)0);
    // enable FIT1 & Video Int
    (*(volatile unsigned int *)(0x80000038)) = 0x00010080;
    microblaze_enable_interrupts();
    fit_status = 0;
    video_status = 0;
    kb = (volatile char *)(KEY_BUFFER);
    for (i=0;i<640;i++) kb[i] = 0;
    uh_init();
    uh_mouse_attach_func(on_mouse);
    uh_keyboard_attach_func(on_keyboard);
    uh_gamepad_attach_func(on_gamepad);
    if (uh_msd_available(0)) {
      get_bitmap_data("FONT.BMP");
    } else xil_printf("MSD not found. FONT.BMP not loaded\n");
    video_init();
    // HID loop
    while(1) {
        uh_update();
        video_update();
    }
    return 0;
}

